import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Button, Modal, notification, message, Progress } from 'antd'
import JSZip from 'jszip'
import { saveAs } from 'file-saver'

import Utils from '@/utils/utils.js'
import Api from '@/api'
import MKEmitter from '@/utils/events.js'
import MkIcon from '@/components/mk-icon'

import './index.scss'

class FuncZip extends Component {
  static propTpyes = {
    btn: PropTypes.object,            // 按钮
    disabled: PropTypes.any,          // 行按钮禁用
  }

  state = {
    loading: false,
    disabled: false,
    loadingNumber: '',
    loadingTotal: '',
    hidden: false,
    visible: false
  }

  UNSAFE_componentWillMount () {
    const { btn, selectedData, BData, disabled } = this.props

    if (btn.controlField) {
      this.setStatus(btn, selectedData || [], BData, disabled)
    } else if (disabled) {
      this.setState({disabled: true})
    }
  }

  componentDidMount () {
    MKEmitter.addListener('triggerBtnId', this.actionTrigger)
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { btn } = this.props

    if (btn.controlField) {
      this.setStatus(btn, nextProps.selectedData || [], nextProps.BData, nextProps.disabled)
    } else {
      this.setState({disabled: nextProps.disabled === true})
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
  }

  setStatus = (btn, data, BData, disprop) => {
    let disabled = false
    let hidden = false

    if (btn.control !== 'parent') {
      if (data.length > 0) {
        data.forEach(item => {
          let s = item[btn.controlField] !== undefined ? item[btn.controlField] + '' : ''
          if (btn.controlVals.includes(s)) {
            disabled = true
          }
        })
      } else if (btn.controlVals.includes('')) {
        disabled = true
      }
    } else {
      if (!BData || !BData.hasOwnProperty(btn.controlField)) {
        hidden = true
      } else {
        let s = BData[btn.controlField] + ''
        if (btn.controlVals.includes(s)) {
          hidden = true
        }
      }
    }

    if (disabled && btn.control === 'hidden') {
      hidden = true
    }

    if (disprop) {
      disabled = true
    }

    this.setState({hidden, disabled})
  }

  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId, record, type) => {
    const { BID, btn, selectedData, setting } = this.props
    const { loading, disabled } = this.state

    if (loading || disabled) return
    if (triggerId && btn.uuid !== triggerId) return

    if (setting.supModule && !BID) {
      notification.warning({
        top: 92,
        message: '需要上级主键值！',
        duration: 5
      })
      return
    } else if (type === 'linkbtn' && !btn.$toolbtn && !is(fromJS(selectedData || []), fromJS(record))) {
      return
    }

    let data = record || selectedData || []
    
    if (btn.Ot !== 'notRequired' && data.length === 0) {
      notification.warning({
        top: 92,
        message: '请选择行！',
        duration: 5
      })
      return
    } else if (btn.Ot === 'requiredSgl' && data.length !== 1) {
      notification.warning({
        top: 92,
        message: '请选择单行数据！',
        duration: 5
      })
      return
    }

    this.setState({
      loading: true
    })

    if (btn.innerFunc) {
      let params = []
      let param = {
        func: btn.innerFunc,
        BID: BID || ''
      }

      if (btn.Ot === 'notRequired') {
        params.push(param)
      } else if (btn.Ot === 'requiredSgl') {
        param.ID = data[0].$$uuid || ''
        params.push(param)
      } else if (btn.Ot === 'required') {
        params = data.map(item => {
          return {
            ...param,
            ID: item.$$uuid || ''
          }
        })
      } else if (btn.Ot === 'requiredOnce') {
        param.ID = data.map(item => item.$$uuid || '').filter(Boolean).join(',')
        params.push(param)
      }

      this.getInnerData(params)
    } else {
      this.downloadZipImage(data, btn.urlkey).then((res) => {
        if (res) {
          this.execError({ErrCode: res})
        } else {
          this.execSuccess()
        }
      }, (err) => {
        this.execError({ErrCode: err})
      })
    }
  }

  getInnerData = (params) => {
    let param = params.shift()

    Api.genericInterface(param).then(res => {
      if (res.status) {
        this.downloadZipImage(res.data, this.props.btn.urlkey).then((res) => {
          if (params.length === 0) {
            if (res) {
              this.execError({ErrCode: res})
            } else {
              this.execSuccess()
            }
          } else {
            this.getInnerData(params)
          }
        }, (err) => {
          if (params.length === 0) {
            this.execError({ErrCode: err})
          } else {
            this.getInnerData(params)
          }
        })
      } else {
        this.execError(res)
      }
    })
  }

  downloadZipImage = (imgArr, imgKey = '') => {
    let images = []
    if (imgArr && imgArr.length > 0) {
      let names = []
      let imgs = []
      imgArr.forEach(item => {
        let itemImg = imgKey ? item[imgKey] : item
        
        if (!itemImg || !/^data:image|jpg$|jpeg$|png$|gif$/.test(itemImg)) return
        if (imgs.includes(itemImg)) return

        let name = Utils.getguid() + '.png'

        if (/.(jpg|jpeg|png|gif)$/.test(itemImg)) {
          let _name = itemImg.replace(/.*\//ig, '')
          if (!names.includes(_name)) {
            name = _name
          }
        }

        imgs.push(itemImg)
        names.push(name)

        images.push({url: itemImg, name: name})
      })
    }
    
    if (images.length === 0) {
      return Promise.reject('01')
    }

    const zip = new JSZip()
    const imgFolder = zip.folder('images') // 创建images文件夹

    return new Promise((resolve, reject) => {
      let deffers = images.map((img) => {
        return new Promise(resolve => {
          this.getBase64(img.url).then((base64) => {
            if (base64) {
              base64 = base64.split('base64,')[1]
              imgFolder.file(img.name, base64, {
                base64: true
              })
              resolve(true)
            } else {
              resolve(false)
            }
          })
        })
      })
  
      Promise.all(deffers).then((res) => {
        let hasSuccess = res.filter((m) => m).length > 0
        let hasError = res.filter((m) => !m).length > 0

        if (hasSuccess) {
          zip.generateAsync({
            type: 'blob'
          }).then((blob) => {
            saveAs(blob, Utils.getguid() + '.zip')
            if (hasError) {
              resolve('02')
            } else {
              resolve('')
            }
          })
        } else {
          reject('03')
        }
      })
    })
  }

  /**
   * 将图片转换成base64 并返回路径
   * @param img
   * @param {number} width 调用时传入具体像素值，控制大小 ,不传则默认图像大小
   * @param {number} height
   * @returns {string}
   */
  getBase64Image = (img, width = 0, height = 0) => {
    const canvas = document.createElement('canvas')
    canvas.width = width ? width : img.width
    canvas.height = height ? height : img.height

    const ctx = canvas.getContext('2d')
    ctx.drawImage(img, 0, 0, canvas.width, canvas.height)
    const dataURL = canvas.toDataURL()
    return dataURL
  }

  /**
   * 加载图片 加载成功后经图片返回
   * @param img
   * @returns {Promise<any>}
   */
  getBase64 = (img) => {
    if (/^data:image/.test(img)) {
      return Promise.resolve(img)
    } else {
      const image = new Image()
      image.crossOrigin = '*'
      image.src = img
      return new Promise((resolve, reject) => {
        image.onload = () => {
          let base64 = ''
          try {
            base64 = this.getBase64Image(image)
          } catch (e) {
            base64 = ''
          }
          resolve(base64)
        }
        image.onerror = () => {
          resolve('')
        }
      })
    }
  }

  /**
   * @description 操作成功后处理
   * 1、excel导出，成功后取消导出按钮加载中状态
   * 2、状态码为 S 时，显示成功信息后系统默认信息
   * 3、状态码为 -1 时，不显示任何信息
   * 4、模态框执行成功后是否关闭
   * 5、通知主列表刷新
   */
  execSuccess = () => {
    const { btn } = this.props

    this.setState({
      loading: false,
      loadingNumber: '',
      loadingTotal: ''
    })
    
    if (btn.execSuccess !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execSuccess, btn)
    }

    if (btn.execSuccess === 'popclose' && btn.$tabId) { // 标签关闭刷新
      MKEmitter.emit('refreshPopButton', btn.$tabId)
    }
  }

  /**
   * @description 操作失败后处理
   */
  execError = (res) => {
    const { btn } = this.props

    this.setState({
      loading: false,
      loadingNumber: '',
      loadingTotal: ''
    })

    if (res.ErrCode === '01') {
      message.error('未获取到下载文件。')
      return
    } else if (res.ErrCode === '02') {
      Modal.error({
        title: '部分文件下载失败：1、请检查文件路径是否正确，2、请检查文件是否跨域。',
      })
      return
    } else if (res.ErrCode === '03') {
      Modal.error({
        title: '文件下载失败：1、请检查文件路径是否正确，2、请检查文件是否跨域。',
      })
      return
    } else if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || res.ErrMesg,
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || res.ErrMesg,
        duration: 10
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || res.ErrMesg,
        duration: 10
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || res.ErrMesg)
    }

    if (btn.execError !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execError, btn)
    }

    if (btn.execError === 'popclose' && btn.$tabId) { // 标签关闭刷新
      MKEmitter.emit('refreshPopButton', btn.$tabId)
    }
  }

  render() {
    const { btn } = this.props
    const { loading, disabled, hidden, loadingNumber, loadingTotal } = this.state

    if (hidden) return null

    let label = ''
    let icon = ''
    let type = 'link'
    let className = ''

    if (btn.show === 'button') {
      label = btn.label
      icon = btn.icon || ''
    } else if (btn.show === 'link') {
      label = <span>{btn.label}{btn.icon ? <MkIcon style={{marginLeft: '8px'}} type={btn.icon}/> : ''}</span>
      icon = ''
    } else if (btn.show === 'icon') {
      icon = btn.icon || ''
    } else if (!btn.$toolbtn) {
      icon = btn.icon || ''
      label = btn.label
      className = 'mk-btn mk-' + btn.class
    } else {
      type = ''
      icon = btn.icon || ''
      label = btn.label
      className = 'mk-btn mk-' + btn.class
    }
    
    if (loadingNumber && !loadingTotal && btn.$toolbtn && (!btn.show || btn.show === 'button')) {
      label = (loadingNumber && !loadingTotal ? `(${loadingNumber})` : '') + btn.label
    }

    return (
      <>
        <Button
          type={type}
          title={disabled ? (btn.reason || '') : (btn.show === 'icon' ? btn.label : '')}
          loading={loading}
          disabled={disabled}
          style={btn.style}
          icon={icon}
          className={className}
          onClick={(e) => {e.stopPropagation(); this.actionTrigger()}}
        >{label}</Button>
        {loadingTotal ? <Progress className="mk-button-progress" percent={(loadingTotal - loadingNumber) / loadingTotal * 100} size="small" showInfo={false} /> : null}
      </>
    )
  }
}

export default FuncZip